unit Kern;                                               (* Version  12.12.00 *)

interface

uses Classes,WinTypes, Graphics, SysUtils, Controls, WinProcs,
     Eingabe, {$IFDEF Graf} FensterGraf {$ELSE} Fenster {$ENDIF}, GIF;

type TSprache = (keine,deutsch,englisch);
     Sprachstring = array [TSprache] of String;

var Sprache: TSprache;

procedure initProgram(inBitmap, Infotext, Startbild: boolean;
                      Fontname: String; Fontsize, Fontstyle: integer);

(************************** allgemeine Routinen *******************************)

procedure wandleZahlInString(r, Format: real; var s: String);

procedure wandleStringInInteger(s: String; var i, Code: integer);

procedure wandleStringInReal(s: String; var r: real; var Code: integer);

function DateiVorhanden(Name: String): boolean;

procedure warte(ms: integer);

(****************************** Tastatur-Routinen *****************************)

const F1 = 1; F2 = 2; f3 = 3; F4 = 4; F5 = 5; F6 = 6; F7 = 7; F8 = 8;
      F9 = 9; F10 = 10; F11 = 11; F12 = 12;
      Esc = 27;
      Return = 23;
      CursorLinks = 13; CursorRechts = 14; CursorHoch = 15; CursorRunter = 16;
      Pos1 = 17; Ende = 18;
      BildHoch = 19; BildRunter = 20;
      Einfg = 21; Entf = 22;
      ClrHome = Entf; Insert = Einfg;       (* nur aus Kompatibilittsgrnden *)
      ShiftF1 = F1+27; ShiftF2 = F2+27; ShiftF3 = F3+27;  ShiftF4 =  F4+27;  ShiftF5 =  F5+27;  ShiftF6 =  F6+27;
      ShiftF7 = F7+27; ShiftF8 = F8+27; ShiftF9 = F9+27; ShiftF10 = F10+27; ShiftF11 = F11+27; ShiftF12 = F12+27;
      GradC = #176;

function Tastencode: integer;  (* liefert 0, wenn keine Taste gedrckt wurde! *)

function TasteGedrueckt: boolean;

procedure liesString(s: String; var Eingabestring: String);

procedure liesChar(s: String; var c: Char);

(****************************** Maus-Routinen *********************************)

const MausAngeschlossen = true;             (* nur aus Kompatibilittsgrnden *)

procedure zeigeMaus;

procedure versteckeMaus;

procedure holeMauszustand(var xMaus, yMaus, MTasten: integer);

procedure setzeMausNach(x, y: integer);

procedure setzeMausform(nr: integer);

(******************************** Akustik-Routinen ****************************)

procedure initTon;                          (* nur aus Kompatibilittsgrnden *)

procedure erzeugeTon(Frequenz, ms: integer);
                         (* funktioniert nur ab Delphi 2.0 und mit Soundkarte *)

procedure schalteTonAus;                    (* nur aus Kompatibilittsgrnden *)

(********************************** Bildschirm-Routinen ***********************)

procedure setzeTextmodus;                   (* nur aus Kompatibilittsgrnden *)

var farbig, SVGA, Vesa, Vesa256: boolean;   (* nur aus Kompatibilittsgrnden *)
    Vesamodus: integer;

procedure setzeGrafikmodus;                 (* nur aus Kompatibilittsgrnden *)

procedure loescheBildschirm;

procedure sichereBildschirm;

procedure rekonstruiereBildschirm;

procedure speichereFBild(Name: String; px1, py1, px2, py2: integer);

procedure speichereGIF(Name: String; x0, y0, x1, y1: integer);

procedure zeichneFBild(Name: String; x0, y0: integer);

procedure speichereSWBild(Name: String; x1, y1, x2, y2: integer);

procedure zeichneSWBild(Name: String; x0, y0:integer);

(**************************** Grafik-Routinen *********************************)

var xmMax, ymMax: integer;               (* max. Breite und Hhe des Fensters *)
    zHoehe, zBreite: integer;
    weiss, schwarz, blau, gruen, cyan, rot, magenta, braun,
    hellgrau, grau, hellblau, hellgruen, hellcyan, hellrot,
    hellmagenta, gelb: integer;

function xm: integer;                             (* akt. Breite des Fensters *)

function ym: integer;                               (* akt. Hhe des Fensters *)

procedure setzePunkt(x, y: integer; Farbe: integer);

function holePunkt(x, y: integer): integer;

type lType = (nichts, durchgehend, langerStrich, Punkte, StrichPunkt,
              Strich, StrichPunktPunkt);

procedure setzeLinienmuster(Muster: lType);

procedure zeichneLinie(x1, y1, x2, y2: integer; Farbe: integer);

var horDir, verDir: Byte;

procedure setzeTextausrichtung(hor, vert: Byte);

procedure setzeTextHFarbe(f: integer);

procedure zeichneGrafstring(x, y: integer; s: String; Farbe: integer);

procedure zeichneVertGrafstring(x, y: integer; s: String; Farbe: integer);

var Randfarbe: integer;

procedure setzeFuelltyp(Helligkeit: integer; Farbe, rFarbe: integer);

procedure fuelleRechteck(x, y, b, h: integer);

procedure fuellePolygon(Eckenzahl: integer; var p: array of TPoint);

procedure setzeVGAPalette;

procedure setzeGrauPalette;

function lambda2Farbe(lambda: real): integer;

function Farbe2Lambda(nr: integer): real;

procedure setzeSpektrumpalette;

procedure setzenSpektrumpalette;

procedure setzeRegenbogenPalette;

procedure setzenRegenbogenPalette;

procedure setzeFarbPalette;

procedure setzeRGBPalette;

(****************************** Sonstiges *************************************)

type TProc = procedure;

var Start, Run: TProc;
    Programmname, Fenstertitel, InfoStr, Info1Str, Info2Str,
    EingabeInfoStr: TCaption;
    zCanvas: TCanvas;

procedure dummyProzedur;

type TFontStyle = set of Graphics.TFontStyle;

procedure waehleFont(Name: String; Size, Style: integer);

procedure zeichneDrehstring(x, y, winkel: integer; s: String; Farbe: integer);

procedure aktualisiereGrafik;              (* um den Bildschirm aufzufrischen *)

procedure GotoXY(x, y: integer);

procedure Write(s: String);

procedure WriteLn(s: String);

implementation

(************************** allgemeine Routinen *******************************)

type rgbquad = array [0..3] of Byte;      (* blau, grn, rot, 0 *)
     RGBFarben = array [0..AnzFarben] of rgbquad;

var RGBPalette: RGBFarben;

procedure wandleZahlInString(r, Format: real; var s: String);

   var vor, nach: integer;

   begin
   vor:=Trunc(Format); nach:=Round(10.0*(Format-vor));
   if nach>0
      then Str(r:vor+nach+2:nach, s)
      else Str(Round(r):vor, s)
   end;

procedure wandleStringInInteger(s: String; var i, Code: integer);

   begin
   Val(s, i, Code)
   end;

procedure wandleStringInReal(s: String; var r: real; var Code: integer);

   begin
   Val(s, r, Code)
   end;

function DateiVorhanden(Name: String): boolean;

   var f: File;

   begin
   Assign(f, Name);
   {$I-} Reset(f); {$I+}
   if IOResult=0
      then
         begin
         Close(f); DateiVorhanden:=Name<>''
         end
      else DateiVorhanden:=false
   end;

procedure warte(ms: integer);

   const cms = 24.0*3600.0*1000.0;

   var t0: real;

   begin
   t0:=Time*cms;
   repeat until Time*cms-t0>ms;
   end;

(****************************** Tastatur-Routinen *****************************)

function Tastencode: integer;

   var t: integer;

   begin
   if Length(KeyBuffer)>0
      then
         begin
         t:=Ord(KeyBuffer[1]);
         Delete(KeyBuffer, 1, 1);
         Tastencode:=t;
         end
      else Tastencode:=0
   end;

function TasteGedrueckt: boolean;

   begin
   TasteGedrueckt:=Length(KeyBuffer)>0;
   end;

procedure liesString(s: String; var Eingabestring: String);

   var ls, les, zBreite: integer;

   begin
   zBreite:=15;
   with EingabeDialog do
      begin
      Hint:=EingabeInfoStr;
      ShowHint:=Hint<>'';
      Width:=344;
      ls:=Label1.Canvas.TextWidth(s);
      les:=Length(Eingabestring)*zBreite;
      if ls>Width-70 then Width:=ls+70;
      with Edit do
         if Width<les+70
            then Width:=les+70
            else if Width>3*les then Width:=3*les;
      if Width<Edit.Width+70 then Width:=Edit.Width+70;
      OKBtn.Left:=(Width-OKBtn.Width-EscBtn.Width) div 3;
      EscBtn.Left:=2*OKBtn.Left+OKBtn.Width;
      Edit.Width:=Length(Eingabestring)*zBreite;
      if Edit.Width<10*zBreite then Edit.Width:=10*zBreite;
      Edit.Left:=(Width-Edit.Width) div 2;
      Edit.Clear;
      Edit.Text:=Eingabestring;
      with Label1 do
         begin
         Caption:=s;
         Width:=Canvas.TextWidth(s);
         Left:=(Eingabedialog.Width-Width) div 2;
         end;
      Edit.Text:=Eingabestring;
      Left:=Grundfenster.ClientOrigin.x+(Grundfenster.ClientWidth-Width) div 2;
      Top:=Grundfenster.ClientOrigin.y+(Grundfenster.ClientHeight-Height) div 2;
      if Sprache=deutsch then Caption:='Eingabe' else Caption:='Input';
      ShowModal;
      Eingabestring:=Edit.Text;
      end;
   end;

procedure liesChar(s: String; var c: Char);

   var sc: String;

   begin
   sc:=c;
   liesString(s, sc);
   c:=sc[Length(sc)]
   end;

(****************************** Maus-Routinen *********************************)

var Mausform: integer;
    MausSichtbar: boolean;

procedure zeigeMaus;

   var p: TPoint;

   begin
   Grundfenster.GrafikBMP.Cursor:=Mausform;
   MausSichtbar:=true;
   getCursorPos(p);                              (* fr sofortige Wirksamkeit *)
   setCursorPos(p.x, p.y);
   end;

procedure versteckeMaus;

   var p: TPoint;

   begin
   Grundfenster.GrafikBMP.Cursor:=crNone;
   MausSichtbar:=false;
   getCursorPos(p);                              (* fr sofortige Wirksamkeit *)
   setCursorPos(p.x, p.y);
   end;

procedure holeMauszustand(var xMaus, yMaus, MTasten: integer);

   begin{$IFDEF Graf}
   xMaus:=FensterGraf.xMaus; yMaus:=FensterGraf.yMaus; MTasten:=FensterGraf.MTasten
   {$ELSE}
   xMaus:=Fenster.xMaus; yMaus:=Fenster.yMaus; MTasten:=Fenster.MTasten
   {$ENDIF}
   end;

procedure setzeMausNach(x, y: integer);

   begin
   SetCursorPos(x, y)
   end;

procedure setzeMausform(nr: integer);

   var p: TPoint;

   begin
   case nr of
      0: Mausform:=crDefault;
      1: Mausform:=crCross;
      2: Mausform:=crHourGlass
      end;
   if MausSichtbar then Grundfenster.GrafikBMP.Cursor:=Mausform;
   getCursorPos(p);                              (* fr sofortige Wirksamkeit *)
   setCursorPos(p.x, p.y);
   end;

(******************************** Akustik-Routinen ****************************)

procedure initTon;

   begin
   dummyProzedur
   end;

procedure erzeugeTon(Frequenz, ms: integer);

   begin
   {$IFDEF Ver20}
   Beep(Frequenz, ms);
   {$ELSE}
   dummyProzedur;
   {$ENDIF}
   Beep(Frequenz, ms);
   end;

procedure schalteTonAus;

   begin
   dummyProzedur
   end;

(********************************** Bildschirm-Routinen ***********************)

var Grafikmodus: boolean;

procedure setzeTextmodus;

   begin
   Grafikmodus:=false
   end;

procedure setzeGrafikmodus;

   begin
   Grafikmodus:=true;
   end;

procedure loescheBildschirm;

(*   Grundfenster.GrafikloeschenClick(Grundfenster)*)
   var r: TRect;
       c: TColor;
       ps: TPenStyle;

   begin
   Fenstertitel:='';
   r.TopLeft:=Point(0, 0);
   r.BottomRight:=Point(xmMax, ymMax);
   with zCanvas do
      begin
      c:=Brush.Color;
      ps:=Pen.Style;
      Brush.Color:=(*clWhite;*)Grundfenster.Color;
      Pen.Style:=psSolid;
      FillRect(r);
      Brush.Color:=c;
      Pen.Style:=ps
      end
   end;

procedure sichereBildschirm;

   begin
   sichern;
   end;

procedure rekonstruiereBildschirm;

   begin
   restaurieren;
   end;

procedure speichereFBild(Name: String; px1, py1, px2, py2: integer);

   begin
   savePicture(Name, px1, py1, px2, py2)
   end;

procedure zeichneFBild(Name: String; x0, y0: integer);

   begin
   loadPicture(Name, x0, y0)
   end;

procedure speichereSWBild(Name: String; x1, y1, x2, y2: integer);

   begin
   savePicture(Name, x1, y1, x2, y2)
   end;

procedure zeichneSWBild(Name: String; x0, y0:integer);

   begin
   loadPicture(Name, x0, y0)
   end;

(**************************** Grafik-Routinen *********************************)

var Fuellfarbe: longint;
    FuellPenStyle: TPenStyle;

function xm: integer;

   begin
   xm:=Grundfenster.ClientWidth
   end;

function ym: integer;

   begin
   {$IFDEF Graf}
   ym:=Grundfenster.ClientHeight;
   {$ELSE}
   ym:=Grundfenster.ClientHeight-Grundfenster.Panel1.Height
   {$ENDIF}
   end;

procedure setzePunkt(x, y: integer; Farbe: integer);

   begin
   if Farbe>AnzFarben then Farbe:=AnzFarben else if Farbe<0 then Farbe:=0;
   with zCanvas do
      begin
(*      Pixels[x, y]:=ColorToRGB(Color[Farbe]);*)
      Pixels[x, y]:=Color[Farbe];
      end
   end;

function holePunkt(x, y: integer): integer;

   var c: TColor;
   f: integer;

   begin
   c:=zCanvas.Pixels[x, y];
   f:=0;
   while (Color[f]<>c) and (f<AnzFarben) do Inc(f);
   holePunkt:=f
   end;

procedure setzeLinienmuster(Muster: lType);

   begin
   with zCanvas.Pen do
   case Muster of
      durchgehend: Style:=psSolid;
      Punkte: Style:=psDot;
      StrichPunkt: Style:=psDashDot;
      StrichPunktPunkt: Style:=psDashDotDot;
      langerStrich, Strich: Style:=psDash;
      end;
      end;

procedure zeichneLinie(x1, y1, x2, y2: integer; Farbe: integer);

   begin
   with zCanvas do
      begin
      Pen.Color:=Color[Farbe];
      PolyLine([Point(x1, y1), Point(x2, y2)]);
      end;
   end;

procedure setzeTextausrichtung(hor, vert: Byte);

   begin
   horDir:=hor; verDir:=vert
   end;

var TextHFarbe: integer;

procedure setzeTextHFarbe(f: integer);

   begin
   TextHFarbe:=f
   end;

procedure zeichneGrafstring(x, y: integer; s: String; Farbe: integer);

   var px, py, h, w: integer;
       c: TColor;

   begin
   with zCanvas do
      begin
      c:=Brush.Color;
      {$IFDEF Graf}
      if c<>FensterGraf.Color[TextHFarbe] then Brush.Color:=FensterGraf.Color[TextHFarbe];
      {$ELSE}
      if c<>Fenster.Color[TextHFarbe] then Brush.Color:=Fenster.Color[TextHFarbe];
      {$ENDIF}
      w:=TextWidth(s); h:=TextHeight(s);
      Font.Color:=Color[Farbe];
      case horDir of
         0: px:=x;
         1: px:=x-w div 2;
         2: px:=x-w
         end;
      case verDir of
         0: py:=y-h;
         1: py:=y-h div 2;
         2: py:=y
         end;
      TextOut(px, py, s);
      Brush.Color:=c;
      end;
   end;

procedure zeichneVertGrafstring(x, y: integer; s: String; Farbe: integer);

   var c: TColor;

   begin
   with zCanvas.Brush do
      begin
      c:=Color;
      {$IFDEF Graf}
      if c<>FensterGraf.Color[TextHFarbe] then Color:=FensterGraf.Color[TextHFarbe];
      {$ELSE}
      if c<>Fenster.Color[TextHFarbe] then Color:=Fenster.Color[TextHFarbe];
      {$ENDIF}
      zeichneDrehString(x, y, 90, s, Farbe);
      Color:=c
      end
   end;

procedure setzeFuelltyp(Helligkeit: integer; Farbe, rFarbe: integer);

   var f, r, g, b: longint;

   begin
   Helligkeit:=Round(1.15*Helligkeit);
   if Helligkeit>255 then Helligkeit:=255;
   f:=ColorToRGB(Color[Farbe]);
   b:=f shr 16;
   g:=(f and $FF00) shr 8;
   r:=f and $FF;
   b:=Round(b/255*Helligkeit);
   g:=Round(g/255*Helligkeit);
   r:=Round(r/255*Helligkeit);
   Fuellfarbe:=b*65536+g*256+r;

   with zCanvas do
      begin
      Brush.Color:=Fuellfarbe;
      if rFarbe=Farbe
         then FuellPenStyle:=psClear
         else FuellPenStyle:=psSolid;
      end;
   Randfarbe:=rFarbe;
   end;

procedure fuelleRechteck(x, y, b, h: integer);

   var r: TRect;
       s: TPenStyle;
       c: TColor;

   begin
   r.Left:=x; r.Top:=y; r.Right:=x+b; r.Bottom:=y+h;
   with zCanvas do
      begin
      s:=Pen.Style;
      c:=Brush.Color;
      Pen.Style:=FuellPenStyle;
      Brush.Color:=Fuellfarbe;
      FillRect(r);
      Pen.Style:=psSolid;
      if FuellPenStyle=psSolid then
         begin
         with r do
            begin
            zeichneLinie(Left, Top, Right, Top, Randfarbe);
            zeichneLinie(Right, Top, Right, Bottom, Randfarbe);
            zeichneLinie(Right, Bottom, Left, Bottom, Randfarbe);
            zeichneLinie(Left, Bottom, Left, Top, Randfarbe);
            end
         end;
      Pen.Style:=s;
      Brush.Color:=c;
      end;
   end;

procedure fuellePolygon(Eckenzahl: integer; var p: array of TPoint);

   var c: TColor;
       ps: TPenStyle;

   begin
   with zCanvas do
      begin
      c:=Brush.Color;
      ps:=Pen.Style;
      Brush.Color:=Fuellfarbe;
      Pen.Style:=FuellPenStyle;
      (*$IFDEF VER20*)
         Polygon(Slice(p, Eckenzahl));
      (*$ELSE*)
         Polygon(p);
      (*$ENDIF*)
      Brush.Color:=c;
      Pen.Style:=ps;
      end
   end;

(****************************** Sonstiges *************************************)

procedure dummyProzedur;

   begin
   end;

procedure waehleFont(Name: String; Size, Style: integer);

   var s: TFontStyle;

   begin
   s:=[];
   if Style=1 then s:=s+[fsbold];
   with zCanvas do
      begin
      Font.Name:=Name;
      Font.Size:=Size;
      Font.Style:=s;
      zBreite:=TextWidth('T'); zHoehe:=TextHeight('T');
      end;
   end;

procedure zeichneDrehString(x, y, Winkel: integer; s: String; Farbe: integer);

   const Grad = Pi/180.0;

   var dc: hdc;
       hFont, fontOld, groesse: integer;
       strint: string;
       charly, txt: array [0..250] of char;
       Weight, w, h: integer;
       c: TColor;
       sinw, cosw: real;

   begin
   sinw:=sin(Winkel*Grad); cosw:=cos(Winkel*Grad);
   strpcopy(txt, s);
   with zCanvas do
      begin
      c:=Brush.Color;
      {$IFDEF Graf}
      if c<>FensterGraf.Color[TextHFarbe] then
         Brush.Color:=FensterGraf.Color[TextHFarbe];
      {$ELSE}
      if c<>Fenster.Color[TextHFarbe] then
         Brush.Color:=Fenster.Color[TextHFarbe];
      {$ENDIF}
      dc:=handle;
      w:=TextWidth(s); h:=TextHeight(s);
      (*$IFDEF VER20*)
         groesse:=Round(Font.Size*1.6);
      (*$ELSE*)
         groesse:=Round(Font.Size*1.6(*9*));
      (*$ENDIF*)
      strint:=Font.Name;
      if fsBold in Font.Style then Weight:=fw_bold else Weight:=fw_normal;
      end;
(*   case verDir of
      0: y:=y;
      1: y:=y+w div 2;
      2: y:=y+w
      end;
   case horDir of
      0: x:=x-h;
      1: x:=x-h div 2;
      2: x:=x
      end;*)
   case horDir of
      1: begin
         x:=x-Round(0.5*w*cosw); y:=y+Round(0.5*w*sinw);
         end;
      2: begin
         x:=x-Round(w*cosw);     y:=y+Round(w*sinw)
         end;
      end;
   case verDir of
      0: begin
         y:=y-Round(h*cosw); x:=x-Round(h*sinw)
         end;
      1: begin
         y:=y-Round(0.5*h*cosw); x:=x-Round(0.5*h*sinw)
         end;
      end;
   strpcopy(charly, strint);
   hfont:=CreateFont(groesse, 0, Winkel*10, 1, Weight, 0, 0, 0,
                     1, 4, $10, 2, 4, charly);
   fontOld:=SelectObject(dc, hfont);
   SetTextColor(dc, Color[Farbe]);
   TextOut(dc, x, y, txt, strlen(txt));
   selectObject(dc, FontOld);
   deleteObject(hFont);
   zCanvas.Brush.Color:=c;
   end;

var i: integer;

procedure aktualisiereGrafik;

   begin
   refresh;
   end;

var xT, yT: integer;

procedure GotoXY(x, y: integer);

   begin
   xT:=x; yT:=y
   end;

procedure Write(s: String);

   var w: integer;

   begin
   w:=zCanvas.TextWidth(s);
   if xT+w>xm then
      begin
      xT:=zBreite; Inc(yT, zHoehe);
      end;
   zeichneGrafstring(xT, yT+zHoehe, s, schwarz);
   Inc(xT, w);
   end;

procedure WriteLn(s: String);

   begin
   zeichneGrafstring(xT, yT, s, schwarz);
   Inc(xT, zCanvas.TextWidth(s));
   xT:=zBreite;
   if yT<ymMax then Inc(yT, zHoehe)
   end;

procedure setRGBPalette(nr, f, r, g, b: integer);

   begin
   (* Der folgende Umweg ist ntig fr den Fall, dass der Monitor nicht auf
      True Color eingestellt ist: Dann werden tatschlich hnliche Farben
      ausgesucht. *)
   zCanvas.Pixels[0,0]:=f*r+ (f*g) shl 8 + (f*b) shl 16;
   {$IFDEF Graf}
   FensterGraf.Color[nr]:=zCanvas.Pixels[0,0];
   {$ELSE}
   Fenster.Color[nr]:=zCanvas.Pixels[0,0];
   {$ENDIF}
   RGBPalette[nr][3]:=0;
   RGBPalette[nr][2]:=f*r;
   RGBPalette[nr][1]:=f*g;
   RGBPalette[nr][0]:=f*b;
   end;

procedure setzeVGAPalette;

   type TPrimaryColors = (Red, Green, Blue);
                      (* Die Farbe in 8-Bit-Komponentenform. *)
        TGifColor = array [TPrimaryColors] of Byte;

   const ftTurbo: array [0..15] of TGifColor =
                                               (( 0,  0,  0),
                                                ( 0,  0, 40),
                                                ( 0, 40,  0),
                                                ( 0, 40, 40),
                                                (40,  7,  7),
                                                (40,  0, 40),
                                                (40, 30,  0),
                                                (49, 49, 49),
                                                (26, 26, 26),
                                                ( 0,  0, 63),
                                                ( 9, 63,  9),
                                                ( 0, 63, 63),
                                                (63, 10, 10),
                                                (44,  0, 63),
                                                (63, 63, 18),
                                                (63, 63, 63));

   var i: integer;

   begin
   for i:=0 to AnzFarben do
      begin
      setRGBPalette(i, 4, ftTurbo[i mod 16][  red],
                          ftTurbo[i mod 16][green],
                          ftTurbo[i mod 16][ blue]);
      end;
   weiss:=15; schwarz:=0;
   blau:=1;gruen:=2;cyan:=3;rot:=4;magenta:=5;
   braun:=6;hellgrau:=7;grau:=8;hellblau:=9;hellgruen:=10;
   hellcyan:=11;hellrot:=12;hellmagenta:=13;gelb:=14;
   setRGBPalette(15, 1, 255, 255, 255);
   setzeTextHFarbe(weiss);
   end;

procedure setzeGrauPalette;

   var k: integer;

   begin
   for k:=0 to 255 do setRGBPalette(k, 1, k, k, k);
   schwarz:=0; weiss:=255;
   setzeTextHFarbe(weiss);
   end;

const lambdaMin = 380.0; lambdaMax = 780.0;

function lambda2Farbe(lambda: real): integer;

   var nr: integer;

   begin
   nr:=Round((lambda-lambdaMin)/(lambdaMax-lambdaMin)*Anzfarben);
   if nr<0
      then nr:=0
      else if nr>AnzFarben-3 then nr:=0;
   lambda2Farbe:=nr;
   end;

function Farbe2Lambda(nr: integer): real;

   var r: real;

   begin
   r:=lambdaMin+(lambdaMax-lambdaMin)/AnzFarben*nr;
   Farbe2Lambda:=r;
   end;

procedure setzeSpektrumpalette;

   const f0 = 0; maxFarbe = 255-f0; maxh = maxFarbe;
         l0 = 390.0; l1 = 440.0; l2 = 475.0; l3 = 510.0;
         l4 = 580.0; l5 = 665.0;

   var i, r, g, b: integer;
       lambda: real;

   function fr(lambda: real): real;

      begin
      fr:=1.0;
      end;

   function fg(lambda: real): real;

      begin
      fg:=1.0;
      end;

   function fb(lambda: real): real;

      begin
      fb:=1.0;
      end;

   begin
   for i:=0 to AnzFarben do
      begin
      lambda:=Farbe2Lambda(i);
      if lambda<l0
         then
            begin
            r:=Trunc(fr(lambda)*maxh*(lambda-lambdaMin)/(l0-lambdaMin));
            g:=Trunc(fg(lambda)*f0);
            b:=Trunc(fb(lambda)*maxh*(lambda-lambdaMin)/(l0-lambdaMin));
            end else
      if lambda<l1
         then
            begin
            r:=Trunc(fr(lambda)*(maxh-maxh*(lambda-l0)/(l1-l0)));
            g:=Trunc(fg(lambda)*f0);
            b:=Trunc(fb(lambda)*maxh);
            end
         else if lambda<l2
            then
               begin
               r:=Trunc(fr(lambda)*f0);
               g:=Trunc(fg(lambda)*maxh*(lambda-l1)/(l2-l1));
               b:=Trunc(fb(lambda)*maxh);
               end
            else if lambda<l3
               then
                  begin
                  r:=Trunc(fr(lambda)*f0);
                  g:=Trunc(fg(lambda)*maxh);
                  b:=Trunc(fb(lambda)*(maxh-maxh*(lambda-l2)/(l3-l2)));
                  end
               else if lambda<l4
                  then
                     begin
                     r:=Trunc(fr(lambda)*maxFarbe*(lambda-l3)/(l4-l3));
                     g:=Trunc(fg(lambda)*(maxh+(maxFarbe-maxh)*(lambda-l3)/(l4-l3)));
                     b:=Trunc(fb(lambda)*f0);
                     end
                  else if lambda<l5
                     then
                        begin
                        r:=Trunc(fr(lambda)*maxFarbe);
                        g:=Trunc(fg(lambda)*(maxFarbe-maxFarbe*(lambda-l4)/(l5-l4)));
                        b:=Trunc(fb(lambda)*f0);
                        end
                     else
                        begin
                        r:=Trunc(fr(lambda)*(maxFarbe-maxFarbe*(lambda-l5)/(lambdaMax-l5)));
                        g:=Trunc(fg(lambda)*f0);
                        b:=Trunc(fb(lambda)*f0);
                        end;
      setRGBPalette(i, 1, r+f0, g+f0, b+f0);
      end;
   i:=0; r:=0; g:=0; b:=0;
   setRGBPalette(i, 1, r, g, b); schwarz:=0;
   i:=253; r:=4*30; g:=4*30; b:=4*30;
   setRGBPalette(i, 1, r, g, b); grau:=253;
   i:=254; r:=4*45; g:=4*45; b:=4*45;
   setRGBPalette(i, 1, r, g, b); hellgrau:=254;
   i:=255; r:=255; g:=255; b:=255;
   setRGBPalette(i, 1, r, g, b); weiss:=255;
   setzeTextHFarbe(weiss);
   end;

procedure setzenSpektrumpalette;

   const f0 = 5; maxFarbe = 255-f0; maxh = 127;
         l1 = 440.0; l2 = 475.0; l3 = 510.0; l4 = 580.0; l5 = 645.0;

   var i, r, g, b: integer;
       lambda, k, fu, fo: real;

   begin
   for i:=0 to AnzFarben do
      begin
      lambda:=Farbe2Lambda(i);
      if lambda<l1
         then
            begin
            fu:=maxh; fo:=f0;  k:=ln(fo/fu);
            r:=Round(fu*exp(k*(lambda-lambdaMin)/(l1-lambdaMin)));
            g:=f0;
            b:=maxh;
            end
         else if lambda<l2
            then
               begin
               r:=f0;
               fu:=f0; fo:=maxh; k:=ln(fo/fu);
               g:=Round(fu*exp(k*(lambda-l1)/(l2-l1)));
               b:=maxh;
               end
            else if lambda<l3
               then
                  begin
                  r:=f0;
                  g:=maxh;
                  fu:=maxh; fo:=f0; k:=ln(fo/fu);
                  b:=Round(fu*exp(k*(lambda-l2)/(l3-l2)));
                  end
               else if lambda<l4
                  then
                     begin
                     fu:=f0; fo:=maxFarbe; k:=ln(fo/fu);
                     r:=Round(fu*exp(k*(lambda-l3)/(l4-l3)));
                     fu:=maxh; fo:=maxFarbe; k:=ln(fo/fu);
                     g:=Round(fu*exp(k*(lambda-l3)/(l4-l3)));
                     b:=f0;
                     end
                  else if lambda<l5
                     then
                        begin
                        r:=maxFarbe;
                        fu:=maxFarbe; fo:=f0; k:=ln(fo/fu);
                        g:=Round(fu*exp(k*(lambda-l4)/(l5-l4)));
                        b:=f0;
                        end
                     else
                        begin
                        fu:=maxFarbe; fo:=f0; k:=ln(fo/fu);
                        r:=Round(fu*exp(k*(lambda-l5)/(lambdaMax-l5)));
                        g:=f0;
                        b:=f0;
                        end;
      setRGBPalette(i, 1, r+f0, g+f0, b+f0);
      end;
   i:=0; r:=0; g:=0; b:=0; schwarz:=0;
   setRGBPalette(i, 1, r, g, b);
   i:=255; r:=255; g:=255; b:=255; weiss:=255;
   setRGBPalette(i, 1, r, g, b);
   setzeTextHFarbe(weiss);
   end;

procedure setzeRegenbogenPalette;

   var k: integer;

   begin
   for k:=0 to 49 do
      setRGBPalette(k, 4, 0, 0, k+14);
   for k:=0 to 49 do
      setRGBPalette(50+k, 4, 0, k+14, 63-k);
   for k:=0 to 49 do
      setRGBPalette(100+k, 4, k+14, 63, 0);
   for k:=0 to 49 do
      setRGBPalette(150+k, 4, 63, 63-k, 0);
   for k:=0 to 55 do
      setRGBPalette(200+k, 4, 63, k+8, k+8);
   setRGBPalette(0, 1, 0, 0, 0); schwarz:=0;
   setRGBPalette(255, 1, 255, 255, 255); weiss:=255;
   setzeTextHFarbe(weiss);
   end;

procedure setzenRegenbogenPalette;

   var k: integer;

   begin
   for k:=0 to 31 do
      setRGBPalette(k, 4, 32-k, 0, 32);
   for k:=0 to 31 do
      setRGBPalette(32+k, 4, 0, k, 32);
   for k:=0 to 31 do
      setRGBPalette(64+k, 4, 0, 32, 32-k);
   for k:=0 to 63 do
      setRGBPalette(96+k, 4, k, 32+k div 2, 0);
   for k:=0 to 63 do
      setRGBPalette(160+k, 4, 63, 63-k, 0);
   for k:=0 to 31 do
      setRGBPalette(224+k, 4, 63-2*k, 0, 0);
   setRGBPalette(0, 1, 0, 0, 0); schwarz:=0;
   setRGBPalette(255, 1, 255, 255, 255); weiss:=255;
   setzeTextHFarbe(weiss);
   end;

procedure setzeFarbPalette;

   var k: integer; r: real;

   begin
   r:=64/36;
   for k:=1 to 36 do (* Blau-Werte *)
      setRGBPalette(k, 4, 0, 0, Trunc(r*k));
   for k:=1 to 36 do (* Rot-Werte *)
      setRGBPalette(36+k, 4, Trunc(r*k), 0, 0);
   for k:=1 to 36 do (* Grn-Werte *)
      setRGBPalette(72+k, 4, 0, trunc(r*k), 0);
   for k:=1 to 36 do
      setRGBPalette(108+k, 4, trunc(r*k), trunc(r*k), 0);
   for k:=1 to 36 do
      setRGBPalette(144+k, 4, trunc(r*k), 0, trunc(r*k));
   for k:=1 to 36 do
      setRGBPalette(180+k, 4, 0, trunc(r*k), trunc(r*k));
   r:=64/39;
   for k:=1 to 39 do (* Grau-Werte *)
      setRGBPalette(216+k, 4, trunc(r*k), trunc(r*k), trunc(r*k));
   setRGBPalette(0, 1, 0, 0, 0); schwarz:=0;
   setRGBPalette(255, 1, 255, 255, 255); weiss:=255;
   setzeTextHFarbe(weiss);
   end;

procedure setzeRGBPalette;

   var k: integer;

   begin
   for k:=0 to 63 do (* Grauwerte *)
      setRGBPalette(k, 4, k, k, k);
   for k:=0 to 63 do (* Rot-Werte *)
      setRGBPalette(64+k, 4, k, 0, 0);
   for k:=0 to 63 do (* Grn-Werte *)
      setRGBPalette(128+k, 4, 0, k, 0);
   for k:=0 to 63 do (* Blau-Werte *)
      setRGBPalette(192+k, 4, 0, 0, k);
   setRGBPalette(0, 1, 0, 0, 0); schwarz:=0;
   setRGBPalette(63, 1, 255, 255, 255); weiss:=63;
   setzeTextHFarbe(weiss);
   end;

procedure speichereGIF(Name: String; x0, y0, x1, y1: integer);

   var j: integer;

   begin
   for j:=0 to Anzfarben do
      begin
      Farbtabelle[j][  red]:=RGBPalette[j,2];
      Farbtabelle[j][green]:=RGBPalette[j,1];
      Farbtabelle[j][ blue]:=RGBPalette[j,0];
      end;
   GIF.holePunkt:=Kern.holePunkt;
   GIFTabelleDefiniert:=true;
   speicheregGIF(Name, x0, y0, x1, y1);
   end;

procedure initProgram(inBitmap, Infotext, Startbild: boolean;
                      Fontname: String; Fontsize, Fontstyle: integer);

   begin
   waehleFont(Fontname, Fontsize, Fontstyle);

   (* Durch den folgenden Aufruf kann dafr gesorgt werden, dass alle Grafiken von
   vornherein in eine Bitmap gezeichnet werden: *)
   {$IFNDEF Graf}
   if inBitmap then Grundfenster.GrafikInBitmapClick(Grundfenster);
   {$ENDIF}
   loescheBildschirm;

   (* Durch den folgenden Aufruf kann dafr gesorgt werden, dass zu Beginn ein
   Informationstext ("(Programmname).txt") angezeigt wird: *)
   if Infotext then zeigeTextdatei(InfoDatei, 10);

   (* Durch den folgenden Aufruf kann dafr gesorgt werden, dass ein Startbild
   ("(Programmname).bmp") gezeigt wird: *)
   if Startbild then loadPicture(InfoBild, 0, 0);

   end;

initialization

Grafikmodus:=true;
Mausform:=crCross; MausSichtbar:=true;
horDir:=0; verDir:=0;

farbig:=true;
(*setzeVGAPalette;*) (* MUSS NUN NACH PROGRAMMINITIALISIERUNG GESCHEHEN !!! *)

Fuellfarbe:=$FFFFFF; FuellPenStyle:=psSolid;
Randfarbe:=weiss;
TextHFarbe:=weiss;

Programmname:=ParamStr(0);
i:=Length(Programmname);
while Programmname[i]<>'.' do Dec(i);
Programmname:=Copy(Programmname, 1, i-1);
i:=Length(Programmname);
while Programmname[i]<>'\' do Dec(i);
Delete(Programmname, 1, i);

Fenstertitel:='';
InfoStr:=''; Info1Str:=''; Info2Str:=''; EingabeInfoStr:='';

Sprache:=deutsch;
end.
