unit RaytraceDelphi;

interface

uses Kern, Modul, Mathe;

type TFarbe = (fweiss, frot, fgruen, fblau);

type Projektion = procedure(r: RVektor; var px, py: integer);

type Gerade = record                                  (* r0+x*e *)
                 r0, e: RVektor;
                 definiert: boolean;
                 end;
     Ebene = record
                Farbe: TFarbe;
                m,                                  (* Aufpunkt *)
                e1, e2,                              (* Basis,  *)
                n: RVektor;      (* Normalenvektor (nach auen) *)
                          (* (e1,e2,n) bilden ein Rechtssystem. *)
                d: real;      (* Abstand der Ebene vom Ursprung *)
                definiert: boolean;
                end;
     Rechteck = record
                   Farbe: TFarbe;
                   m,                            (* Mittelpunkt *)
                   e1, e2,                (* Orthonormalbasis,  *)
                   n: RVektor;   (* Normalenvektor (nach auen) *)
                          (* (e1,e2,n) bilden ein Rechtssystem. *)
                   d,         (* Abstand der Ebene vom Ursprung *)
                   a1, a2: real;          (* halbe Seitenlngen *)
                   definiert: boolean;
                   end;
     Dreieck = record
                  Farbe: TFarbe;
                  d0,           (* Startpunkt der Basisvektoren *)
                  b1, b2,                      (* Basisvektoren *)
                  e1, e2,                   (* Orthonormalbasis *)
                  n: RVektor;    (* Normalenvektor (nach auen) *)
                          (* (e1,e2,n) bilden ein Rechtssystem. *)
                  b21, b22,               (* Komponenten von r2 *)
                  d: real;    (* Abstand der Ebene vom Ursprung *)
                  definiert: boolean;
                  end;
     qRechtecke = array[1..6] of Rechteck;
     Quader = record
                 r: qRechtecke;
                 definiert: boolean
                 end;
     pDreiecke = array[1..4] of Dreieck;
     Pyramide = record                  (* rechteckige Pyramide *)
                   r: Rechteck;
                   d: pDreiecke;
                   definiert: boolean;
                   end;
     Kugel = record
                Farbe: TFarbe;
                m: RVektor;                      (* Mittelpunkt *)
                r: real;                              (* Radius *)
                definiert: boolean;
                end;
     Zylinder = record
                   Farbe: TFarbe;
                   m, e: RVektor;
                   r, a, ru: real;
                   definiert: boolean
                   end;
     Kegel = record
                Farbe: TFarbe;
                sp, m,
                e1, e2, e3: RVektor;
                r, h, rU: real;
                definiert: boolean
                end;

     GRSchnittpunkt = record
                         definiert: boolean;        (* Treffer? *)
                         f: integer;
                         amRand: boolean;
                         s: RVektor;            (* Schnittpunkt *)
                         s1, s2,          (* Flchenkoordinaten *)
                         sp: real;       (* cos(Auftreffwinkel) *)
                         end;
     GESchnittpunkt = record
                         definiert: boolean;        (* Treffer? *)
                         f: integer;
                         s: RVektor;            (* Schnittpunkt *)
                         lambda, sp: real;       (* cos(Auftreffwinkel) *)
                         end;
     GDSchnittpunkt = GRSchnittpunkt;
     GPSchnittpunkt = GRSchnittpunkt;
     GKSchnittpunkte = record
                         definiert: boolean;        (* Treffer? *)
                         f: integer;
                         n: integer; (* Anzahl der Schnittpunkte*)
                         s1,                    (* nchster ... *)
                         s2: RVektor; (* und weiterer Schnittp. *)
                         n1, n2: RVektor;  (* zugeh. Normalenv. *)
                         lambda1, lambda2,
                         sp1, sp2: real;       (* zugeh. cosini *)
                         end;
     GZSchnittpunkte = record
                         definiert: boolean;        (* Treffer? *)
                         f: integer;
                         n: integer; (* Anzahl der Schnittpunkte*)
                         s1,                    (* nchster ... *)
                         s2: RVektor; (* und weiterer Schnittp. *)
                         n1, n2: RVektor;  (* zugeh. Normalenv. *)
                         sp1, sp2: real;       (* zugeh. cosini *)
                         end;
     GKeSchnittpunkte = record
                         definiert: boolean;        (* Treffer? *)
                         f: integer;
                         n: integer; (* Anzahl der Schnittpunkte*)
                         s1,                    (* nchster ... *)
                         s2: RVektor; (* und weiterer Schnittp. *)
                         n1, n2: RVektor;  (* zugeh. Normalenv. *)
                         sp1, sp2: real;       (* zugeh. cosini *)
                         end;

var eps, dRand: real;
    Abbildung: Projektion;

procedure definiereGerade(r1, r2: RVektor; var g: Gerade);

procedure definiereRechteck(P1, P2, P3: RVektor; f: TFarbe;
                            var r: Rechteck);
(* P1, P2, P3 sind drei aufeinander folgende Eckpunkte, die, von
auen betrachtet, einen mathematisch positiven Umlaufsinn
definieren. *)

procedure definiereEbene(P1, P2, P3: RVektor; f: TFarbe;
                         var e: Ebene);
(* P1, P2, P3 sind drei aufeinander folgende Eckpunkte, die, von
oben betrachtet, einen mathematisch positiven Umlaufsinn
definieren. *)

procedure zeichneDrahtrechteck(r: Rechteck; Farbe: integer);

procedure definiereDreieck(P1, P2, P3: RVektor; f: TFarbe;
                           var dr: Dreieck);
(* P1, P2, P3 sind drei aufeinander folgende Eckpunkte, die, von
auen betrachtet, einen mathematisch positiven Umlaufsinn
definieren.*)

procedure zeichneDrahtDreieck(d: Dreieck; Farbe: integer);

procedure definiereQuader(r0, r1, r2, r3: RVektor;
              (* r1, r2, r3 mssen orthog. Rechtssystem bilden. *)
                          f1, f2, f3, f4, f5, f6: TFarbe;
                          var q: Quader);

procedure zeichneDrahtquader(q: Quader; Farbe: integer);

procedure definierePyramide(r0, r1, r2: RVektor;
(* r1 und r2 mssen orthog. sein, werden mit h zu einem
                                        Rechtssystem erweitert. *)
                            h: real;
                            f0, f1, f2, f3, f4: TFarbe;
                            var p: Pyramide);

procedure zeichneDrahtPyramide(p: Pyramide; Farbe: integer);

procedure definiereKugel(m: RVektor; r: real; f: TFarbe;
                         var k: Kugel);

procedure zeichneDrahtkugel(k: Kugel; Farbe: integer);

procedure definiereZylinder(m, Achse: RVektor; f: TFarbe;
                            r, h: real;
                            var z: Zylinder);

procedure zeichneDrahtzylinder(z: Zylinder; Farbe: integer);

procedure definiereKegel(Spitze, Achse: RVektor; f: TFarbe;
                         r, h: real;
                         var k: Kegel);

procedure zeichneDrahtkegel(k: Kegel; Farbe: integer);

procedure berechneGESchnittpunkt(g: Gerade; e: Ebene;
                                 var s: GESchnittpunkt);
(* ... berechnet den Schnittpunkt zwischen der Geraden g und der
                                                       Ebene e. *)

procedure berechneGRSchnittpunkt(g: Gerade; r: Rechteck;
                                 var s: GRSchnittpunkt);
(* ... berechnet den Schnittpunkt zwischen der Geraden g und dem
                                                    Rechteck r. *)

procedure berechneGDSchnittpunkt(g: Gerade; dr: Dreieck;
                                 var s: GDSchnittpunkt);
(* ... berechnet den Schnittpunkt zwischen der Geraden g und dem
                                                    Dreieck dr. *)

procedure berechneGQSchnittpunkt(g: Gerade; q: Quader;
                                 var s: GRSchnittpunkt);

procedure berechneGPSchnittpunkt(g: Gerade; p: Pyramide;
                                 var s: GPSchnittpunkt);

procedure quadrGleichung(a, b, c: real;        (* Koeffizienten *)
                         var n: integer;   (* Anz. der Lsungen *)
                         var x1,            (* kleinere und ... *)
                             x2: real);       (* grere Lsung *)

procedure berechneGKSchnittpunkte(g: Gerade; k: Kugel;
                                   var sp: GKSchnittpunkte);

procedure berechneGZSchnittpunkte(g: Gerade; z: Zylinder;
                                  var s: GZSchnittpunkte);

procedure berechneGKeSchnittpunkte(g: Gerade; k: Kegel;
                                   var s: GKeSchnittpunkte);

procedure berechneReflexionsstrahl(e: Gerade; n: RVektor;
                                   var r: Gerade);

procedure berechneBrechungsstrahl(e: Gerade; n: RVektor;
                                  Brechzahl: real; var b: Gerade);

implementation

procedure definiereGerade(r1, r2: RVektor; var g: Gerade);

   begin
   if r2[0]=0.0 then
      begin
      g.definiert:=false; Exit;
      end;
   with g do
      begin
      r0:=r1;
      e:=r2; normiereRVektor(e);
      definiert:=true;
      end;
   end;

procedure definiereEbene(P1, P2, P3: RVektor; f: TFarbe;
                            var e: Ebene);

   begin
   with e do
      begin
      definiert:=false;
      Farbe:=f;
      m:=P2;
      subtrahiereRVektoren(P3, P2, e1);
      subtrahiereRVektoren(P1, P2, e2);
      if (e1[0]=0.0) or (e2[0]=0.0) then Exit;
      normiereRVektor(e1);
      normiereRVektor(e2);
      if Abs(Skalarprodukt(e1, e2))>0.999 then Exit;
      berechneKreuzprodukt(e1, e2, n);
      normiereRVektor(n);
      d:=Skalarprodukt(m, n);
      definiert:=true;
      end;
   end;

procedure definiereRechteck(P1, P2, P3: RVektor; f: TFarbe;
                            var r: Rechteck);

   begin
   with r do
      begin
      Farbe:=f;
      addiereRVektoren(P1, P3, m);
      multipliziereRVektor(0.5, m);
      subtrahiereRVektoren(P3, P2, e1);
      subtrahiereRVektoren(P1, P2, e2);
      if (e1[0]=0.0) or (e2[0]=0.0) or
         (Abs(Skalarprodukt(e1, e2))>eps) then
         begin
         definiert:=false; Exit;
         end;
      a1:=0.5*e1[0]; a2:=0.5*e2[0];
      normiereRVektor(e1);
      normiereRVektor(e2);
      berechneKreuzprodukt(e1, e2, n);
      d:=Skalarprodukt(m, n);
      definiert:=true;
      end;
   end;

procedure zeichneDrahtrechteck(r: Rechteck; Farbe: integer);

   var P: array [1..4] of RVektor;
       x, y: array[1..4] of integer;
       i: integer;

   begin
   with r do
      begin
      if not definiert then Exit;
      multipliziereRVektor(a1, e1);
      multipliziereRVektor(a2, e2);
      addiereRVektoren(m, e1, P[1]);
      addiereRVektoren(P[1], e2, P[1]);
      multipliziereRVektor(2.0, e1);
      multipliziereRVektor(2.0, e2);
      subtrahiereRVektoren(P[1], e1, P[2]);
      subtrahiereRVektoren(P[2], e2, P[3]);
      addiereRVektoren(P[3], e1, P[4]);
      end;
   for i:=1 to 4 do Abbildung(P[i], x[i], y[i]);
   geheNach(x[4], y[4]);
   for i:=1 to 4 do zieheLinieNach(x[i], y[i], Farbe);
   end;

procedure zeichneDrahtDreieck(d: Dreieck; Farbe: integer);

   var x0, y0, x, y: integer;

   begin
   Abbildung(d.d0, x0, y0); geheNach(x0, y0);
   addiereRVektoren(d.d0, d.b1, d.b1);
   Abbildung(d.b1, x, y); zieheLinieNach(x, y, Farbe);
   addiereRVektoren(d.d0, d.b2, d.b2);
   Abbildung(d.b2, x, y); zieheLinieNach(x, y, Farbe);
   zieheLinieNach(x0, y0, Farbe);
   end;

procedure definiereDreieck(P1, P2, P3: RVektor; f: TFarbe;
                           var dr: Dreieck);

   begin
   with dr do
      begin
      Farbe:=f;
      d0:=P2;
      subtrahiereRVektoren(P3, P2, b1);
      subtrahiereRVektoren(P1, P2, b2);
      berechneKreuzprodukt(b1, b2, n);
      if (b1[0]=0.0) or (b2[0]=0.0) or (n[0]<eps) then
         begin
         definiert:=false; Exit;
         end;
      normiereRVektor(n);
      e1:=b1; normiereRVektor(e1);
      berechneKreuzprodukt(n, e1, e2);
      b21:=Skalarprodukt(b2, e1);
      b22:=Skalarprodukt(b2, e2);
      d:=Skalarprodukt(d0, n);
      definiert:=true;
      end;
   end;

procedure definiereQuader(r0, r1, r2, r3: RVektor;
                          f1, f2, f3, f4, f5, f6: TFarbe;
                          var q: Quader);

   var r, P1, P2, P3, P4, P5, P6, P7, P8: RVektor;

   begin
   q.definiert:=false;
   if Abs(Skalarprodukt(r1, r2)*Skalarprodukt(r2, r3)*
          Skalarprodukt(r3, r1))>eps then Exit;
   berechneKreuzprodukt(r1, r2, r);
                 (* r1, r2, r3 mssen ein Rechtssystem bilden ! *)
   if Skalarprodukt(r, r3)<0.0 then Exit;
   q.definiert:=true;
   P1:=r0;
   addiereRVektoren(P1, r1, P2);
   addiereRVektoren(P1, r3, P4);
   addiereRVektoren(P1, r2, P5);
   addiereRVektoren(P5, r1, P6);
   addiereRVektoren(P5, r3, P8);
   addiereRVektoren(P2, r3, P3);
   addiereRVektoren(P6, r3, P7);
   definiereRechteck(P2, P1, P5, f1, q.r[1]);
   definiereRechteck(P5, P1, P4, f2, q.r[2]);
   definiereRechteck(P4, P1, P2, f3, q.r[3]);
   definiereRechteck(P8, P4, P3, f4, q.r[4]);
   definiereRechteck(P2, P6, P7, f5, q.r[5]);
   definiereRechteck(P8, P7, P6, f6, q.r[6]);
   end;

procedure zeichneDrahtquader(q: Quader; Farbe: integer);

   var i: integer;

   begin
   if not q.definiert then Exit;
   for i:=1 to 6 do zeichneDrahtrechteck(q.r[i], Farbe);
   end;

procedure definierePyramide(r0, r1, r2: RVektor;
                            h: real;
                            f0, f1, f2, f3, f4: TFarbe;
                            var p: Pyramide);

   var m, sp, P0, P1, P2, P3: RVektor;

   begin
   p.definiert:=false;
   if Abs(Skalarprodukt(r1, r2))>eps then Exit;
   p.definiert:=true;
   addiereRVektoren(r1, r2, m); multipliziereRVektor(0.5, m);
   addiereRVektoren(r0, m, m);
   berechneKreuzprodukt(r1, r2, sp); normiereRVektor(sp);
   multipliziereRVektor(h, sp);
   addiereRVektoren(m, sp, sp);
   P0:=r0;
   addiereRVektoren(P0, r1, P1);
   addiereRVektoren(P0, r2, P2);
   addiereRVektoren(P1, r2, P3);
   definiereRechteck(P1, P0, P2, f0, p.r);
   definiereDreieck(P0, P1, sp, f1, p.d[1]);
   definiereDreieck(P2, P0, sp, f2, p.d[2]);
   definiereDreieck(P3, P2, sp, f3, p.d[3]);
   definiereDreieck(P1, P3, sp, f4, p.d[4]);
   end;

procedure zeichneDrahtpyramide(p: Pyramide; Farbe: integer);

   var i: integer;

   begin
   for i:=1 to 4 do zeichneDrahtdreieck(p.d[i], Farbe);
   end;

procedure definiereKugel(m: RVektor; r: real; f: TFarbe;
                         var k: Kugel);

   begin
   if r<=0.0 then
      begin
      k.definiert:=false; Exit;
      end;
   k.Farbe:=f;
   k.m:=m; k.r:=r; k.definiert:=true;
   end;

procedure zeichneDrahtkugel(k: Kugel; Farbe: integer);

   var rl: RVektor;
       phi, alpha, r, r1, r2: real;
       x, y: integer;

   begin
   phi:=0.0*Grad;
   while phi<180.0*Grad do
      begin
      phi:=phi+45.0*Grad; r1:=k.r*cos(phi); r2:=k.r*sin(phi);
      alpha:=0.0;
      rl[1]:=k.m[1]+r1*cos(alpha);
      rl[2]:=k.m[2]+r2*cos(alpha);
      rl[3]:=k.m[3]+k.r*sin(alpha);
      Abbildung(rl, x, y); geheNach(x, y);
      while alpha<370.0*Grad do
         begin
         rl[1]:=k.m[1]+r1*cos(alpha);
         rl[2]:=k.m[2]+r2*cos(alpha);
         rl[3]:=k.m[3]+k.r*sin(alpha);
         Abbildung(rl, x, y);
         zieheLinieNach(x, y, Farbe);
         alpha:=alpha+10.0*Grad;
         end;
      end;
   phi:=-60.0*Grad;
   while phi<89.0*Grad do
      begin
      phi:=phi+30.0*Grad; r:=k.r*cos(phi);
      rl[3]:=k.m[3]+k.r*sin(phi);
      alpha:=0.0;
      rl[1]:=k.m[1]+r*sin(alpha);
      rl[2]:=k.m[2]+r*cos(alpha);
      Abbildung(rl, x, y); geheNach(x, y);
      while alpha<370.0*Grad do
         begin
         rl[1]:=k.m[1]+r*sin(alpha);
         rl[2]:=k.m[2]+r*cos(alpha);
         Abbildung(rl, x, y);
         zieheLinieNach(x, y, Farbe);
         alpha:=alpha+10.0*Grad;
         end;
      end;
   end;

procedure definiereZylinder(m, Achse: RVektor; f: TFarbe;
                            r, h: real;
                            var z: Zylinder);

   begin
   z.Farbe:=f;
   z.m:=m;
   z.e:=Achse; normiereRVektor(z.e);
   z.r:=r; z.a:=0.5*h;
   z.ru:=sqrt(sqr(r)+sqr(z.a));
   z.definiert:=true;
   end;

procedure zeichneDrahtzylinder(z: Zylinder; Farbe: integer);

   var e2, e3, rl, r2, r3, l, P: RVektor;
       i, j, x, y, xp, yp: integer;
       alpha: real;

   begin
   l:=z.e; multipliziereRVektor(2.0*z.a, l);
   rl:=z.e; multipliziereRVektor(-z.a, rl);
   addiereRVektoren(z.m, rl, rl);
   with z do definiereRVektor(e[2], e[3], e[1], e2);
   berechneKreuzprodukt(z.e, e2, e2);
   multipliziereRVektor(z.r, e2);
   berechneKreuzprodukt(z.e, e2, e3);
   multipliziereRVektor(z.r, e3);
   multipliziereRVektor(0.5*z.a, z.e);
   for j:=1 to 5 do
      begin
      i:=0; alpha:=i*10.0*Grad;
      r2:=e2; multipliziereRVektor(cos(alpha), r2);
      r3:=e3; multipliziereRVektor(sin(alpha), r3);
      addiereRVektoren(r2, r3, r3);
      addiereRVektoren(rl, r3, r3); Abbildung(r3, x, y);
      geheNach(x, y);
      while i<36 do
         begin
         Inc(i);
         alpha:=i*10.0*Grad;
         r2:=e2; multipliziereRVektor(cos(alpha), r2);
         r3:=e3; multipliziereRVektor(sin(alpha), r3);
         addiereRVektoren(r2, r3, r3);
         addiereRVektoren(rl, r3, r3);Abbildung(r3, x, y);
         zieheLinieNach(x, y, Farbe);
         if j=1 then
            if i mod 6 = 0 then
               begin
               addiereRVektoren(r3, l, P);
               Abbildung(P, xp, yp);
               zeichneLinie(x, y, xp, yp, Farbe);
               end;
         end;
      addiereRVektoren(rl, z.e, rl);
      end;
   end;

procedure definiereKegel(Spitze, Achse: RVektor; f: TFarbe;
                         r, h: real;
                         var k: Kegel);

   var s: real;

   begin
   if (Achse[0]=0.0) or (h=0.0) or (r=0.0) then
      begin
      k.definiert:=false; Exit
      end;
   k.definiert:=true;
   k.Farbe:=f;
   k.h:=h; k.r:=r;
   k.sp:=Spitze;
   k.e1:=Achse; normiereRVektor(k.e1);
   berechneKreuzprodukt(Spitze, k.e1, k.e2);
   if k.e2[0]>0.0
      then normiereRVektor(k.e2)
      else
         begin
         with k do definiereRVektor(e1[2], e1[3], e1[1], e2);
         berechneKreuzprodukt(k.e1, k.e2, k.e2);
         normiereRVektor(k.e2);
         end;
   berechneKreuzprodukt(k.e1, k.e2, k.e3);
   s:=h*h+r*r;
   k.rU:=0.5*s/h;
   k.m:=k.e1; multipliziereRVektor(k.rU, k.m);
   addiereRVektoren(k.sp, k.m, k.m);
   end;

procedure zeichneDrahtkegel(k: Kegel; Farbe: integer);

   var xsp, ysp, x, y, i: integer;
       r1, r2, r3: RVektor;
       r, f, alpha: real;

   begin
   Abbildung(k.sp, xsp, ysp);
   f:=0.33;
   while f<1.1 do
      begin
      r:=f*k.r; r1:=k.e1; multipliziereRVektor(f*k.h, r1);
      addiereRVektoren(k.sp, r1, r1);
      i:=0; alpha:=i*10.0*Grad;
      r2:=k.e2; multipliziereRVektor(r*cos(alpha), r2);
      r3:=k.e3; multipliziereRVektor(r*sin(alpha), r3);
      addiereRVektoren(r2, r3, r3);
      addiereRVektoren(r1, r3, r3);Abbildung(r3, x, y);
      geheNach(x, y);
      while i<36 do
         begin
         Inc(i);
         alpha:=i*10.0*Grad;
         r2:=k.e2; multipliziereRVektor(r*cos(alpha), r2);
         r3:=k.e3; multipliziereRVektor(r*sin(alpha), r3);
         addiereRVektoren(r2, r3, r3);
         addiereRVektoren(r1, r3, r3);Abbildung(r3, x, y);
         zieheLinieNach(x, y, Farbe);
         if f>0.8 then
            if i mod 6 = 0 then
               zeichneLinie(x, y, xsp, ysp, Farbe);
         end;
      f:=f+0.33;
      end;
   end;

procedure berechneGESchnittpunkt(g: Gerade; e: Ebene;
                                 var s: GESchnittpunkt);

   var skp1, skp2, x: real;

   begin
   s.definiert:=false;
   if (not g.definiert) or (not e.definiert) then Exit;
   skp1:=Skalarprodukt(g.e, e.n);
   if skp1=0.0 then Exit;         (* Gerade und Flche parallel *)
   skp2:=Skalarprodukt(g.r0, e.n);
     (* Berechnung des Schnittpunktes s.r nach Stcker, S. 387: *)
   x:=(e.d-skp2)/skp1;
   s.lambda:=x;
   s.s :=g.e; multipliziereRVektor(x, s.s);
   addiereRVektoren(g.r0, s.s, s.s);
     (* Untersuchung, ob Schnittpunkt innerhalb des Rechteckes: *)
   s.sp:=skp1;
   s.definiert:=true;
   s.f:=integer(e.Farbe)*64
   end;

procedure berechneGRSchnittpunkt(g: Gerade; r: Rechteck;
                                 var s: GRSchnittpunkt);

   var skp1, skp2, x: real;
       v: RVektor;

   begin
   if (not g.definiert) or (not r.definiert) then
      begin
      s.definiert:=false; exit;
      end;
   skp1:=Skalarprodukt(g.e, r.n);
   if skp1=0.0 then               (* Gerade und Flche parallel *)
      begin
      s.definiert:=false; exit;
      end;
   skp2:=Skalarprodukt(g.r0, r.n);
     (* Berechnung des Schnittpunktes s.r nach Stcker, S. 387: *)
   x:=(r.d-skp2)/skp1;
   s.s :=g.e; multipliziereRVektor(x, s.s);
   addiereRVektoren(g.r0, s.s, s.s);
     (* Untersuchung, ob Schnittpunkt innerhalb des Rechteckes: *)
   v:=r.m;
   subtrahiereRVektoren(s.s, v, v);
                   (* v=s.r-r.m: Schnittpunkt als Flchenvektor *)
   s.s1:=Skalarprodukt(v, r.e1);
   if Abs(s.s1)>r.a1 then
      begin
      s.definiert:=false; Exit;
      end;
   s.s2:=Skalarprodukt(v, r.e2);
   if Abs(s.s2)>r.a2 then
      begin
      s.definiert:=false; Exit;
      end;
   s.sp:=skp1;
   s.amRand:=(Abs(s.s1)>dRand*r.a1) or (Abs(s.s2)>dRand*r.a2);
   s.definiert:=true;
   s.f:=integer(r.Farbe)*64
   end;

procedure berechneGQSchnittpunkt(g: Gerade; q: Quader;
                                 var s: GRSchnittpunkt);

   var i: integer;
       smin: GRSchnittpunkt;

   begin
   s.s[0]:=1.0e10; s.definiert:=false; s.sp:=1.0;
   i:=1;
   repeat
      berechneGRSchnittpunkt(g, q.r[i], smin);
      if smin.definiert then
         if smin.s[0]<s.s[0] then s:=smin;
      Inc(i);
   until (i=7) or (s.sp<0.0)
   end;

procedure berechneGPSchnittpunkt(g: Gerade; p: Pyramide;
                                 var s: GPSchnittpunkt);

   var i: integer;
       smin: GRSchnittpunkt;

   begin
   s.s[0]:=1.0e10; s.definiert:=false; s.sp:=1.0;
   berechneGRSchnittpunkt(g, p.r, smin);
   if smin.definiert then
      if smin.s[0]<s.s[0] then s:=smin;
   if s.sp<0.0 then Exit;
   i:=1;
   repeat
      berechneGDSchnittpunkt(g, p.d[i], smin);
      if smin.definiert then
         if smin.s[0]<s.s[0] then s:=smin;
      Inc(i);
   until (i=5) or (s.sp<0.0)
   end;

procedure berechneGDSchnittpunkt(g: Gerade; dr: Dreieck;
                                 var s: GDSchnittpunkt);

   var skp1, skp2, ve1, ve2: real;
       v: RVektor;

   begin
   if (not g.definiert) or (not dr.definiert) then
      begin
      s.definiert:=false; exit;
      end;
   skp1:=Skalarprodukt(g.e, dr.n);
   if skp1=0.0 then               (* Gerade und Flche parallel *)
      begin
      s.definiert:=false; exit;
      end;
   skp2:=Skalarprodukt(g.r0, dr.n);
     (* Berechnung des Schnittpunktes s.r nach Stcker, S. 387: *)
   s.s:=g.e; multipliziereRVektor((dr.d-skp2)/skp1, s.s);
   addiereRVektoren(g.r0, s.s, s.s);
     (* Untersuchung, ob Schnittpunkt innerhalb des Rechteckes: *)
   v:=dr.d0;
   subtrahiereRVektoren(s.s, v, v);
                 (* v=s.r-dr.d0: Schnittpunkt als Flchenvektor *)
   ve1:=Skalarprodukt(v, dr.e1);           (* Komponenten von v *)
   ve2:=Skalarprodukt(v, dr.e2);               (* bzgl. (e1,e2) *)
          (* s.r1, s.r2: Komponenten von v bzgl. (dr.b1,dr.b2): *)
   s.s2:=ve2/dr.b22;
   if (s.s2<0.0) or (s.s2>1.0) then
      begin
      s.definiert:=false; Exit;
      end;
   s.s1:=(ve1-ve2*dr.b21/dr.b22)/dr.b1[0];
   if (s.s1<0.0) or (s.s1+s.s2>1.0) then
      begin
      s.definiert:=false; Exit;
      end;
   s.sp:=skp1;
   s.amRand:=(s.s1>dRand) or (s.s1<1.0-dRand) or
             (s.s2>dRand) or (s.s2<1.0-dRand) or
             (s.s1+s.s2>dRand);
   s.definiert:=true;
   s.f:=integer(dr.Farbe)*64;
   end;

procedure quadrGleichung(a, b, c: real;
                         var n: integer;
                         var x1, x2: real);

   var rd: real;

   begin
   if a=0.0
      then
         begin
         if b=0.0
            then n:=0
            else
               begin
               n:=1;
               x1:=-c/b;
               end
         end
      else
         begin
         b:=0.5*b/a; c:=c/a;
         rd:=b*b-c;
         if rd<0.0
            then n:=0
            else if rd=0.0
                    then
                       begin
                       n:=1;
                       x1:=-b;
                       end
                    else
                       begin
                       n:=2;
                       rd:=sqrt(rd);
                       x1:=-b-rd;
                       x2:=-b+rd;
                       end;
         end;
   end;

procedure berechneGKSchnittpunkte(g: Gerade; k: Kugel;
                                   var sp: GKSchnittpunkte);

   var r, r1, r2, n1, n2: RVektor;
       a, b, c, x1, x2, sp1, sp2: real;

   begin
   if (not g.definiert) or (not k.definiert) then
      begin
      sp.definiert:=false; sp.n:=0; Exit;
      end;
   subtrahiereRVektoren(k.m, g.r0, r);
   berechneKreuzprodukt(g.e, r, r);
   if r[0]>k.r
      then
         begin
         sp.definiert:=false; sp.n:=0;
         end
      else
         begin
         subtrahiereRVektoren(k.m, g.r0, r);
         a:=sqr(g.e[0]);
         b:=-2.0*Skalarprodukt(g.e, r);
         c:=sqr(r[0])-sqr(k.r);
         quadrGleichung(a, b, c, sp.n, x1, x2);
         sp.definiert:=true;
         sp.f:=integer(k.Farbe)*64;
         r1:=g.e; multipliziereRVektor(x1, r1);
         addiereRVektoren(g.r0, r1, r1);
         subtrahiereRVektoren(r1, k.m, n1); normiereRVektor(n1);
         sp1:=Skalarprodukt(n1, g.e);
         sp.s1:=r1; sp.n1:=n1; sp.sp1:=sp1; sp.lambda1:=x1;
         if sp.n=2 then
            begin
            r2:=g.e; multipliziereRVektor(x2, r2);
            addiereRVektoren(g.r0, r2, r2);
            subtrahiereRVektoren(r2, k.m, n2);
            normiereRVektor(n2);
            sp2:=Skalarprodukt(n2, g.e);
            sp.s2:=r2; sp.n2:=n2;
            sp.sp2:=sp2; sp.lambda2:=x2;
            end;
         end;
   end;

procedure berechneGZSchnittpunkte(g: Gerade; z: Zylinder;
                                  var s: GZSchnittpunkte);

   var r1, r2, a1, a2: RVektor;
       d, Spatprodukt, lambda1, lambda2: real;

   begin
   s.n:=0; s.definiert:=false;
   if (not g.definiert) or (not z.definiert) then Exit;
          (* Abstand d zwischen g und z.m nach Stcker, S. 385: *)
   subtrahiereRVektoren(g.r0, z.m, r1);
   berechneKreuzprodukt(g.e, r1, r2);
   d:=r2[0];
   if (d>z.ru) or (Abs(Skalarprodukt(g.e, z.e))=1) then Exit;
   berechneKreuzprodukt(z.e, g.e, r2);
   Spatprodukt:=Skalarprodukt(r1, r2);
                   (* Abstand zwischen Gerade und Zylinderachse *)
   d:=Abs(Spatprodukt)/r2[0];
   if d>z.r then Exit;
   berechneKreuzprodukt(z.e, r1, r1);
   quadrGleichung(sqr(r2[0]), 2.0*Skalarprodukt(r1, r2),
                  sqr(r1[0])-sqr(z.r), s.n, lambda1, lambda2);
   if s.n=0 then Exit;
   r1:=g.e; multipliziereRVektor(lambda1, r1);
   addiereRVektoren(g.r0, r1, r1);
   subtrahiereRVektoren(r1, z.m, r1);
   r2:=g.e; multipliziereRVektor(lambda2, r2);
   addiereRVektoren(g.r0, r2, r2);
   subtrahiereRVektoren(r2, z.m, r2);
   if r1[0]>z.ru then Dec(s.n);
   if r2[0]>z.ru then Dec(s.n);
   if s.n=0 then Exit;
   s.definiert:=true;
   s.f:=integer(z.Farbe)*64;
   if (s.n=1) and (r1[0]>z.ru) then r1:=r2;
   a1:=z.e; multipliziereRVektor(Skalarprodukt(r1, z.e), a1);
   subtrahiereRVektoren(r1, a1, s.n1); normiereRVektor(s.n1);
   addiereRVektoren(z.m, r1, s.s1);
   s.sp1:=Skalarprodukt(g.e, s.n1);
   if s.n=1 then Exit;
   a2:=z.e; multipliziereRVektor(Skalarprodukt(r2, z.e), a2);
   subtrahiereRVektoren(r2, a2, s.n2); normiereRVektor(s.n2);
   addiereRVektoren(z.m, r2, s.s2);
   s.sp2:=Skalarprodukt(g.e, s.n2);
(*   if abs(lambda1)>abs(lambda2) then
      begin
      r1:=s.s1; s.s1:=s.s2; s.s2:=r1;
      r1:=s.n1; s.n1:=s.n2; s.n2:=r1;
      d:=s.sp1; s.sp1:=s.sp2; s.sp2:=d;
      end;*)
   end;

procedure berechneGKeSchnittpunkte(g: Gerade; k: Kegel;
                                   var s: GKeSchnittpunkte);

   var r, r1, r2: RVektor;
       d, lambda1, lambda2: real;
       a1, a2, a3, b1, b2, b3: real;

   begin
   s.n:=0; s.definiert:=false;
   if (not g.definiert) or (not k.definiert) then Exit;
          (* Abstand d zwischen g und k.m nach Stcker, S. 385: *)
   subtrahiereRVektoren(g.r0, k.m, r1);
   berechneKreuzprodukt(g.e, r1, r2);
   d:=r2[0];
   if d>k.rU then Exit;
   subtrahiereRVektoren(g.r0, k.sp, r1);
   a1:=k.r*Skalarprodukt(r1, k.e1);
   a2:=k.h*Skalarprodukt(r1, k.e2);
   a3:=k.h*Skalarprodukt(r1, k.e3);
   b1:=k.r*Skalarprodukt(g.e, k.e1);
   b2:=k.h*Skalarprodukt(g.e, k.e2);
   b3:=k.h*Skalarprodukt(g.e, k.e3);
   quadrGleichung(sqr(b3)+sqr(b2)-sqr(b1),
                  2.0*(a3*b3+a2*b2-a1*b1),
                  sqr(a3)+sqr(a2)-sqr(a1),
                  s.n, lambda1, lambda2);
   if s.n=0 then Exit;
   s.s1:=g.e; multipliziereRVektor(lambda1, s.s1);
   addiereRVektoren(g.r0, s.s1, s.s1);
   subtrahiereRVektoren(s.s1, k.sp, r1);
   if s.n=2 then
      begin
      s.s2:=g.e; multipliziereRVektor(lambda2, s.s2);
      addiereRVektoren(g.r0, s.s2, s.s2);
      subtrahiereRVektoren(s.s2, k.sp, r2);
      end;
   s.sp1:=Skalarprodukt(r1, k.e1);
   if (s.sp1<0.0) or (s.sp1>k.h) then Dec(s.n);
   s.sp2:=Skalarprodukt(r2, k.e1);
   if (s.sp2<0.0) or (s.sp2>k.h) then Dec(s.n);
   if s.n=0 then Exit;
   s.definiert:=true;
   s.f:=integer(k.Farbe)*64;
   if (s.n=1) and ((s.sp1<0.0) or (s.sp1>k.h)) then
      begin
      s.s1:=s.s2; r1:=r2;
      end;
   berechneKreuzprodukt(k.e1, r1, r);
   berechneKreuzprodukt(r, r1, s.n1); normiereRVektor(s.n1);
   s.sp1:=Skalarprodukt(g.e, s.n1);
   if s.n=1 then Exit;
   berechneKreuzprodukt(k.e1, r2, r);
   berechneKreuzprodukt(r, r2, s.n2); normiereRVektor(s.n2);
   s.sp2:=Skalarprodukt(g.e, s.n2);
(*   if abs(lambda1)>abs(lambda2) then
      begin
      r1:=s.s1; s.s1:=s.s2; s.s2:=r1;
      r1:=s.n1; s.n1:=s.n2; s.n2:=r1;
      d:=s.sp1; s.sp1:=s.sp2; s.sp2:=d;
      end;*)
   end;

procedure berechneReflexionsstrahl(e: Gerade; n: RVektor;
                                   var r: Gerade);

   var h: RVektor;
       sp: real;

   begin
   r.definiert:=false;
   sp:=Skalarprodukt(n, e.e);
   if sp>0.0 then Exit;
   r.definiert:=true;
   r.r0:=e.r0;
   h:=n;
   multipliziereRVektor(2.0*sp, h);
   subtrahiereRVektoren(e.e, h, r.e);
   end;

procedure berechneBrechungsstrahl(e: Gerade; n: RVektor;
                                  Brechzahl: real; var b: Gerade);

   var en, et: RVektor;
       sp: real;

   begin
   b.definiert:=false;
   sp:=Skalarprodukt(n, e.e);
   b.r0:=e.r0;
   en:=n;
   multipliziereRVektor(sp, en); (* Normalkomponente von g.e *)
   subtrahiereRVektoren(e.e, en, et);
   multipliziereRVektor(1.0/Brechzahl, et);
   if et[0]>1.0 then begin write(#7); Exit; end;
   multipliziereRVektor(sqrt(1.0-sqr(et[0]))/en[0], en);
   addiereRVektoren(en, et, b.e);
   b.definiert:=true;
   end;

begin
eps:=1.0e-10; dRand:=0.99;
end.
