unit GIF; (* Version vom 21.9.1999 *)

interface

type TPrimaryColors = (Red, Green, Blue);
                         (* Die Farbe in 8-Bit-Komponentenform. *)
     TGifColor = array [TPrimaryColors] of Byte;
     GIFTabelle = array [0..255] of TGifColor;

var holePunkt: function(x, y: integer): integer;
    Farbtabelle: GIFTabelle;
    GIFTabelleDefiniert: boolean;

procedure speicheregGIF(Name: String; x0, y0, x1, y1: integer);

implementation

const xmMax = 1279; ymMax = 1023;

type Zeile = array [0..xmMax] of Byte;

const minLZWCodeSize: Byte = 8;(* Min LZW code size = 8 (8 bit) *)
      BlockTerminator: byte = 0;
      FileTrailer: byte = $3B;                (* GIF-Terminator *)
      gifBGColor: byte = 0;                 (* Hintergrundfarbe *)
      gifPixAsp: byte = 0; (* PixelAspektRatio (wird ignoriert) *)
      gifcolordepth: byte = 8;            (* 8 bit = 256 colors *)
      hashc = 5101;
      gifLIDid: byte = $2C;                 (* Bildtrennzeichen *)
      HASHSIZE: integer = hashc;
      HASHBITS: integer = 4;
      EMPTY: integer = -1;
      GlobalColorMap: byte = $80;
      SortFlag: byte = 0;

var f: File;
    ImageWidth, ImageHeight: integer;
    buffer: array[0..255] of byte;
    codes: array[0..hashc] of integer;
    prefix: array[0..hashc] of integer;
    suffix: array[0..hashc] of integer;
    nbytes, nbits, cursize, curcode, maxcode: integer;
    minsize, maxsize, nroots, Capacity: integer;
    endc, clrc: integer;
    AnzFarben: integer;
    bytecode, bytemask: integer;
    counter: integer;
    strc, chrc: integer;
    started: boolean;

    x0l, y0l: integer;

procedure ClearX;

   var j: integer;

   begin
   cursize:=minsize;
   nbits:=cursize;
   curcode:=endc+1;
   maxcode:=1 shl cursize;
   for j:=0 to HASHSIZE do codes[j]:=EMPTY;
   end;

procedure Initialisierung;

   var flags, Pixel, ColorResolution : Byte;
       st: String;

   begin
   counter:=0;
   started:=false;
   bytecode:=0;
   bytemask:=1;
   Capacity:=HASHSIZE;
   minsize:=9;
   maxsize:=12;
   nroots:=1 shl 8;
   clrc:=nroots;
   endc:=clrc+1;
   AnzFarben:=1 shl gifcolordepth;
   ClearX;
   st:='GIF87a';
   BlockWrite(f, st[1], 6);                   (* Versionsnummer *)
                            (* Write the GIF screen descriptor: *)
   BlockWrite(f, ImageWidth, 2);
   BlockWrite(f, ImageHeight, 2);
   Pixel:=gifcolordepth-1;
   ColorResolution:=Pixel shl 4;
   flags:= GlobalColorMap or ColorResolution or SortFlag or Pixel;
   BlockWrite(f, flags, 1);                  (* Resolution flag *)
   BlockWrite(f, gifBGColor, 1);
   BlockWrite(f, gifPixAsp, 1);
   end;

procedure schreibeFarbtabelle(var ft: GIFTabelle);

   const ftTurbo: array [0..15] of TGifColor =
                                               (( 0,  0,  0),
                                                ( 0,  0, 40),
                                                ( 0, 40,  0),
                                                ( 0, 40, 40),
                                                (40,  7,  7),
                                                (40,  0, 40),
                                                (40, 30,  0),
                                                (49, 49, 49),
                                                (26, 26, 26),
                                                ( 0,  0, 63),
                                                ( 9, 63,  9),
                                                ( 0, 63, 63),
                                                (63, 10, 10),
                                                (44,  0, 63),
                                                (63, 63, 18),
                                                (63, 63, 63));

   var i: integer;

   begin
   for i:=0 to AnzFarben-1 do
      begin
      ft[i]:=ftTurbo[i mod 16];
      ft[i][  red]:=ft[i][  red] shl 2;
      ft[i][green]:=ft[i][green] shl 2;
      ft[i][ blue]:=ft[i][ blue] shl 2;
      end;
   end;

procedure schreibeImageDescriptor;                    (* S. 255 *)

   var gifxLeft, gifyTop: word;             (* Must be 16 bit!! *)
       (*Pixel, SortFlag, InterlacedFlag, localFlag,*) flags: Byte;

   begin
   BlockWrite(f, gifLIDid, 1);
   gifxLeft:=0; BlockWrite(f, gifxLeft, 2);
   gifyTop :=0; BlockWrite(f, gifyTop,  2);
   BlockWrite(f, ImageWidth, 2);
   BlockWrite(f, ImageHeight, 2);
   flags:=0;
{   localFlag:=0 shl 7;              (* keine lokale Farbtabelle *)
   Pixel:=0;              (* deshalb Anz. der Bit pro Pixel = 0 *)
   SortFlag:=0 shl 5;             (* Farbtabelle nicht sortiert *)
   InterlacedFlag:=0 shl 6;(* sequentielle Anordnung der Zeilen *)}
   BlockWrite(f, flags, 1);
   end;

procedure Putbyte(B: integer);

   begin
   counter:=counter+1;
   buffer[nbytes]:=B;
   Inc(nbytes);
   if nbytes=255 then
     begin
     BlockWrite(f, nbytes, 1);
     BlockWrite(f, buffer, nbytes);
     nbytes:=0;
     end;
   end;

procedure PutCode(code: integer);

   var n, mask: integer;

   begin
   mask:=1;
   n:=nbits;
   while n>0 do
      begin
      Dec(n);
      if (code and mask)<>0 then
         bytecode:=(bytecode or bytemask);
      bytemask:=bytemask shl 1;
      if bytemask>$80 then
         begin
         PutByte(bytecode);
         bytecode:=0;
         bytemask:=1;
         end;
      mask:=mask shl 1;
      end;
   end;

function findstr(pfx, sfx: integer): integer;

   var i, di: integer;

   begin
   i:=(sfx shl HASHBITS) xor pfx;
   if i=0 then di:=1 else di:=Capacity -i;
   while true do
      begin
      if codes[i]=EMPTY then break;
      if (prefix[i] = pfx) and (suffix[i] = sfx) then break;
      i:=i-di;
      if i<0 then i:=i+Capacity;
      end;
   findstr:=i;
   end;

procedure EncodeScanLine(var buf: Zeile; npxls: integer);

   var np, i: integer;

   begin
   np:=0;
   if not Started then
      begin
      strc:=buf[np];
      Inc(np);
      Started:=True;
      end;
   while np<npxls do
      begin
      chrc := buf[np];
      Inc(np);
      i:=findstr(strc, chrc);
      if codes[i]<>EMPTY
         then strc:=codes[i]
         else
            begin
            codes[i]:=curcode;
            prefix[i]:=strc;
            suffix[i]:=chrc;
            putcode(strc);
            strc:=chrc;
            Inc(curcode);
            if curcode>maxcode then
               begin
               Inc(cursize);
               if cursize>maxsize
                  then
                     begin
                     putcode(clrc);
                     ClearX;
                     end
                  else
                     begin
                     nbits:=cursize;
                     maxcode:=maxcode shl 1;
                     if cursize=maxsize then Dec(maxcode);
                     end;
            end;
         end;
      end;
   end;

procedure schreibeBilddaten;

   var m, i: integer;
       scanLine, lZeile: Zeile;

   begin
   BlockWrite(f, MinLZWCodesize, 1);
   PutCode(clrc);
   m:=0;
   while m<ImageHeight do
      begin
      FillChar(scanLine, ImageWidth, 0);
      for i:=0 to ImageWidth-1 do
         lZeile[i]:=holePunkt(x0l+i, y0l+m);
      move(lZeile, scanLine,ImageWidth);
      encodeScanLine(scanLine, ImageWidth);
      Inc(m);
      end;
   end;

procedure Flush;

   begin
   if bytemask<>1 then
     begin
     PutByte(byteCode);
     bytecode :=0;
     bytemask :=1;
     end;
   if nbytes>0 then
      begin
      BlockWrite(f, nbytes, 1);
      BlockWrite(f, buffer, nbytes);
      nbytes:=0;
      end;
   end;

procedure saveGif(Breite, Hoehe: integer;
                  Name : String);

   begin
   Assign(f, Name); Rewrite(f, 1);
   if Breite>xmMax+1 then Breite:=xmMax+1;
   if Hoehe>ymMax+1 then Hoehe:=ymMax+1;
   ImageWidth:=Breite; ImageHeight:=Hoehe;
   Initialisierung;
   if not GIFTabelleDefiniert then
      begin
      schreibeFarbtabelle(Farbtabelle);
      GIFTabelleDefiniert:=true;
      end;
   BlockWrite(f, Farbtabelle, 3*AnzFarben);
   schreibeImageDescriptor;
   schreibeBilddaten;
   PutCode(strc);
   PutCode(endc);
   Flush;
   BlockWrite(f, BlockTerminator, 1);
   BlockWrite(f, FileTrailer, 1);
   Close(f);
   end;

procedure speicheregGIF(Name: String; x0, y0, x1, y1: integer);

   begin
   nBytes:=0;
   x0l:=x0; y0l:=y0;
   ImageWidth:=x1-x0+1;
   if ImageWidth>xmMax+1 then ImageWidth:=xmMax+1;
   ImageHeight:=y1-y0+1;
   if ImageHeight>ymMax+1 then ImageHeight:=ymMax+1;
   saveGIF(ImageWidth, ImageHeight, Name);
   end;

begin
GIFTabelleDefiniert:=false;
end.
